/*
 * Created on 30-mar-2005
 *
 * Vector() version 1.0
 */

/**
 * @version 1.0
 * @author <b>Miguel Rebolledo</b><br>
 * 
 * <b>Tarea de Dise&ntilde;o y Programaci&oacute;n Orientada a Objetos</b><br>
 * La clase Vector implementa un objeto con las cualidades de cualquier
 * vector en el plano RxR, siendo posible implementarlo en 
 * coordenadas polares o cartesianas, ademas consta con metodos de suma
 * de vectores, ya sea devolviendo un tercer vector como es el caso del
 * metodo <i>suma()</i> o tambien el resultado modificando alguno de los 
 * vectores que se estan sumando(<i>sumarle()</i>).<br> 
 * Estas operaciones b&aacute;sicas de vectores se pueden resumir en aquellas entre dos vectores:
 * <ul>
 * <li><i>suma(Vector a, Vector b)</i>
 * <li><i>sumarle(Vector a)</i>
 * <li><i>resta(Vecot a, Vector b)</i>
 * <li><i>productoPunto(Vector a, Vector b)</i>
 *</ul> 
 *Ademas, las operaciones propias en un vector:
 *<ul>
 *<li><i>rotar(desfase)</i>
 *<li><i>productoPorEscalar(escalar)</i>
 *<li><i>equals(Vector b)</i>
 *<li><i>calculaAngulo()</i>
 *<li><i>calculaModulo()</i>
 *<li><i>toString()</i>
 *</ul>
 * Soluci&oacute;n entregada para tarea 1.<br> 
 * de Dise&ntilde;o y Programaci&oacute;n Orientada a Objetos - elo329<br>
 * Universidad T&eacute;cnica Federico Santa Maria<br>
 * Departamento de Electr&oacute;nica<br>
 * Profesor:Agust&iacute;n Gonz&aacute;lez<br>
 * 
 */
public class Vector {
	/**
	 * X componente horizontal del vector
	 */
	protected double X;
	/**
	 * Y componente vertical del vector
	 */
	protected double Y;
	
	/**
	 * Constructor que instancia un vector de coordenadas
	 * cartesianas.
	 * @param x Componente ordenada del vector
	 * @param y Componente absisa del vector
	 */
	Vector(double x, double y)
	{
		X = x;
		Y = y;
	}
	/**
	 *  Contructor que instancia un vector de coordenadas polares.
	 * @param r Radio del vector
	 * @param a Angulo del vector
	 */
	Vector(float r, double a)
	{
		X = r * Math.cos(a);
		Y = r * Math.sin(a);
	}
	/**
	 * Constructor por omisi&oacute;n que instancia un vector cartesiano de
	 * componentes (0,0)<br>
	 * <li> X = 0
	 * <li> Y = 0
	 */
	Vector()
	{
		X = 0;
		Y = 0;
	}
	/**
	 * Metodo que devuelve el angulo del vector respecto a la
	 * ordenada
	 * @return Angulo del vector
	 * @since Vector() 1.0
	 */
	public double calculaAngulo()
	{
		return Math.atan2(X, Y) ;
		}
	/**
	 * Metodo que retorna la Magnitud del vector objeto
	 * @return Modulo del Vector
	 */
	public double calculaMagnitud()
	{
		return Math.sqrt(Math.pow(X, 2)+Math.pow(Y, 2));
	}
	/**
	 * Metodo que retorna un vector con la suma de sus dos argumentos
	 * @param a Vector a
	 * @param b Vector b
	 * @return Vector suma entre a y b
	 */
	public static Vector suma(Vector a, Vector b)
	{
	    Vector Z = new Vector(a.X + b.X, a.Y + b.Y);
	    return Z;
	}
	/**
	 * Metodo que suma al presente objeto el Vector b
	 * @param b vector a sumarle 
	 */
	public void sumarle(Vector b)
	{
	    X = X + b.X;
	    Y = Y + b.Y;
	}
	/**
	 * Metodo que resta dos vectores a y b y devuelve el resultado resultante
	 * @param a Vector a
	 * @param b Vector b
	 * @return Vector C =(Vector a - Vector b)
	 */
	public static Vector resta(Vector a, Vector b)
	{
	    Vector Z = new Vector(a.X - b.X, a.Y - b.Y );
	    return Z;
	}
	/**
	 * Metodo que entrega el producto punto entre dos vectores.<br>
	 * El producto punto entre los vectores A y B se calcula:<br>
	 * <i>A.x * B.x + A.y * B.y</i>
	 * @param a Vector A
	 * @param b Vector B
	 * @return Producto punto entre los vectores a y b.
	 */
	public static double productoPunto(Vector a, Vector b)
	{
	    return a.X * b.X + a.Y * b.Y; 
	}
	/**
	 * Metodo que amplia la magnitud del vector en el factor m
	 * @param m factor de ampliacion
	 */
	public void multiplicacionPorEscalar(double m)
	{
		X = X * m;
		Y = Y * m;
	}
	/**
	 * Metodo que modifica el angulo del vector sumandole el factor <i>d</i> del argumento al angulo que posee.<br>
	 * Esta rotacion angular se realiza sin cambiar la magnitud del vector, por lo tanto quedara cada componente de la siguiente forma:<br>
	 * Sea V vector de coordenadas (<i>X</i>,<i>Y</i>) y angulo <i>theta</i> y magnitud <i>M</i><br>
	 * <i>X = M * Math.cos(theta + d)</i><br>
	 * <i>Y = M * Math.sin(theta + d)</i>  
	 * @param d factor a sumarle al angulo del objeto
	 */
	public void rotar(float d)
	{
		Vector miVector = new Vector(X , Y);
		float D = (float)miVector.calculaAngulo() + d;
		X = this.calculaMagnitud() * Math.cos(D);
		Y = this.calculaMagnitud() * Math.sin(D);
	}
	/**
	 * Metodo que compara el objeto que invoca con el que tiene como argumento
	 * Esta comparaci&oacute;n se realiza componente a componente en coordenadas cartesianas.
	 * La salida de esta comparaci&oacute;n es de tipo boolean 
	 * @param b Vector a comparar
	 * @return True or False
	 */
	public boolean equals(Vector b)
	{
		if((X == b.X)&&(Y == b.Y))
			return true;
		else
			return false;
	}
	/**
	 * Metodo que entrega las coordenadas cartesianas
	 */
	public String toString()
	{
		return "\t Cartesianas ("+(float)X+", "+(float)Y+") |\t Polares ("+(float)this.calculaMagnitud()+", "+(float)Math.atan2(X , Y)+")";
	}

} 